//
//  main.m
//  MetalTextureViews2
//
//  Created by Litherum on 6/21/20.
//  Copyright © 2020 Litherum. All rights reserved.
//

@import Foundation;
@import Metal;
#import <assert.h>

enum Type {
    DoesntRequireFlag,
    RequiresFlag,
    Impossible
};

static enum Type testView(id<MTLDevice> device, MTLPixelFormat textureFormat, MTLPixelFormat viewFormat) {
    MTLTextureDescriptor *textureDescriptor = [MTLTextureDescriptor texture2DDescriptorWithPixelFormat:textureFormat width:2 height:2 mipmapped:NO];
    textureDescriptor.storageMode = MTLStorageModePrivate;
    id<MTLTexture> texture = [device newTextureWithDescriptor:textureDescriptor];
    assert(texture);
    @try {
        [texture newTextureViewWithPixelFormat:viewFormat];
        return DoesntRequireFlag;
    } @catch (NSException *exception) {
        //NSLog(@"%@", exception);
    }

    textureDescriptor.usage |= MTLTextureUsagePixelFormatView;
    texture = [device newTextureWithDescriptor:textureDescriptor];
    assert(texture);
    @try {
        [texture newTextureViewWithPixelFormat:viewFormat];
        return RequiresFlag;
    } @catch (NSException *exception) {
    }

    return Impossible;
}

int main(int argc, const char * argv[]) {
    id<MTLDevice> device = MTLCreateSystemDefaultDevice();
    assert(device);
    MTLPixelFormat formats[] = {
        MTLPixelFormatR8Unorm,
        MTLPixelFormatR8Snorm,
        MTLPixelFormatR8Uint,
        MTLPixelFormatR8Sint,
        MTLPixelFormatR16Uint,
        MTLPixelFormatR16Sint,
        MTLPixelFormatR16Float,
        MTLPixelFormatRG8Unorm,
        MTLPixelFormatRG8Snorm,
        MTLPixelFormatRG8Uint,
        MTLPixelFormatRG8Sint,
        MTLPixelFormatR32Uint,
        MTLPixelFormatR32Sint,
        MTLPixelFormatR32Float,
        MTLPixelFormatRG16Uint,
        MTLPixelFormatRG16Sint,
        MTLPixelFormatRG16Float,
        MTLPixelFormatRGBA8Unorm,
        MTLPixelFormatRGBA8Unorm_sRGB,
        MTLPixelFormatRGBA8Snorm,
        MTLPixelFormatRGBA8Uint,
        MTLPixelFormatRGBA8Sint,
        MTLPixelFormatBGRA8Unorm,
        MTLPixelFormatBGRA8Unorm_sRGB,
        MTLPixelFormatRGB10A2Unorm,
        MTLPixelFormatRG11B10Float,
        MTLPixelFormatRG32Uint,
        MTLPixelFormatRG32Sint,
        MTLPixelFormatRG32Float,
        MTLPixelFormatRGBA16Uint,
        MTLPixelFormatRGBA16Sint,
        MTLPixelFormatRGBA16Float,
        MTLPixelFormatRGBA32Uint,
        MTLPixelFormatRGBA32Sint,
        MTLPixelFormatRGBA32Float,
        MTLPixelFormatDepth32Float,
        MTLPixelFormatDepth32Float_Stencil8,
        MTLPixelFormatStencil8,
    };
    const int formatCount = sizeof(formats) / sizeof(formats[0]);
    enum Type results[formatCount][formatCount];
    for (int i = 0; i < formatCount; ++i) {
        MTLPixelFormat textureFormat = formats[i];
        for (int j = 0; j < formatCount; ++j) {
            MTLPixelFormat viewFormat = formats[j];
            results[i][j] = testView(device, textureFormat, viewFormat);
        }
        printf("\n");
    }
    for (size_t i = 0; i < formatCount; ++i) {
        for (size_t j = 0; j < formatCount; ++j) {
            switch (results[i][j]) {
                case DoesntRequireFlag:
                    printf("free\t");
                    break;
                case RequiresFlag:
                    printf("y\t");
                    break;
                case Impossible:
                    printf("n\t");
                    break;
            }
        }
        printf("\n");
    }
    return 0;
}
